#include <em_device.h>
#include "hardware.h"
#include "lcd.h"
#include "i2c.h"
extern void Delay_ms(uint32_t);
uint8_t LCDbuf[20];

void Delay_ms(uint32_t d)
{
	RTC->COMP0 = 33*d;
	RTC->CTRL = RTC_CTRL_EN;			// start RTC
	while (! (RTC->IF & RTC_IF_COMP0)){}
	RTC->IFC = RTC_IFC_COMP0;			// clear RTC flag
	RTC->CTRL = 0;						// stop RTC		
}

void LCD_setup()
{
	uint8_t initSeq[] = {0x00, 0x38, 0x39, 0x14, 0x7E, 0x5E, 0x6C, 0x0C, 0x06, 0x01};	
	(GPIO->P[1]).DOUTSET = RST;			// release RESET line
	Delay_ms(100); 						// wait after power is applied	
	I2CMasterWriteBuf(LCD_ADDRESS, initSeq, 10); // send init sequence
	Delay_ms(10);
	(GPIO->P[1]).DOUTCLR = LIGHT;		// background light
	NVIC_ClearPendingIRQ(RTC_IRQn);
	NVIC_EnableIRQ(RTC_IRQn);			// enable RTC interrupt in NVIC	
} 

void LCD_topLine(char* line)			// send text to the top LCD line
{
	uint8_t len;						// message length
	LCDbuf[0] = 0x80;					// 1st control byte
	LCDbuf[1] = 0x80;					// set cursor
	LCDbuf[2] = 0x40;					// last control byte
	for (len=0; line[len]; len++)		// add string to buffer
		LCDbuf[len+3] = (uint8_t) line[len];
	I2CMasterWriteBuf(LCD_ADDRESS, LCDbuf, len+3); 
}

void LCD_botLine(char* line)			// send text to the bottom LCD line
{
	uint8_t len;						// message length
	LCDbuf[0] = 0x80;					// 1st control byte
	LCDbuf[1] = 0xC0;					// set cursor
	LCDbuf[2] = 0x40;					// last control byte
	for (len=0; line[len]; len++)				// add string to buffer
		LCDbuf[len+3] = (uint8_t) line[len];
	I2CMasterWriteBuf(LCD_ADDRESS, LCDbuf, len+3);
}


void reactionTimeOut(uint32_t reactionTimeOutput)
{
	
	char time[] = {"         ms "};				// create array of chars
	uint8_t i = 7;												//char array pointer
	while (reactionTimeOutput != 0)
	{
		if (i == 4)								// If the reaction time goes over 1 Second changes the output from mS to Sec notation.
		{
			time[i+3] = time[i+2]; 	//move second decimal value back one position
			time[i+2] = time[i+1]; 	//move first decimal value back one position
			time[i+1] = '.';
			time[9] = 'S';
			time[10] = 'e';
			time[11] = 'c';
		}
		
		uint8_t val = reactionTimeOutput%10;										//retrieve the individual values of reactionTime
		reactionTimeOutput = (reactionTimeOutput - val)/10;		//remove the last value retrieved from reactionTime
		
		char intToString = val+'0';	//convert the integer to a char
		time[i] = intToString;			//insert the individual values into the char array
		
		i--;											// move to the previous array position to insert the number in correct order
	}
	
	LCD_botLine(time);
}

