// Reaction game:
// Wait for LED to light and press BUTTON as soon as you can.
// Watch your reaction time on terminal screen.
// If the button is not pressed within 5 sec, then "You are too slow" will be printed.
// The game repeats over and over.

#include <em_device.h>							// header for all micro registers
#include "hardware.h"							// custom headers
#include "lcd.h"

const uint16_t ran[] = {3,1,5,2,4,1,3,5};		// "random" waiting intervals
volatile uint8_t ran_index = 0;					// waiting index
uint32_t timerCnt, reaction;					// storage for reaction time

int main()
{
	CMU_setup();								// configure hardware
	GPIO_setup();
	PRS_setup();
	Timers_setup();
	RTC_setup();
	I2C0_setup();
	LCD_setup();

	while(1)
	{

		LCD_topLine("     Ready?     ");
		LCD_botLine("  Push button   ");
		__WFI();
		while(!((GPIO->P[1].DIN) & BUTTON)){}	// wait for button release

		LCD_topLine(" Wait for LED to");
		LCD_botLine("press the button");
		RTC->COMP0 = ran[(++ran_index & 7)]*50000; // set random waiting interval
		RTC->COMP1 = RTC->COMP0 + 32768*5;		// set LED 5 sec timeout
		RTC->CTRL = RTC_CTRL_EN;				// start RTC
		do
		{
			__WFI();							// wait for LED turned on and press BUTTON
		}
		while (!((GPIO->P[1]).DIN & LED));		// ignore button presses
		__WFI();								// wait for button press

		if (!((GPIO->P[1]).DIN & BUTTON))		// BUTTON is pressed
		{
			timerCnt = ((TIMER1->CC[1]).CCVB << 16) + (TIMER0->CC[1]).CCVB; // collect captured timestamps
			reaction = (timerCnt*1000) / 109375;							// compute reaction time in ms
			RTC->IFS = RTC_IF_COMP1;										// force to stop RTC and turn off LED
			
			LCD_botLine("                ");
			LCD_topLine("Reaction time is");								// LCD output
			reactionTimeOut(reaction);
			__WFI();							// press button to continue
		}
		else									// LED timeout
		{
			LCD_topLine("You are too slow");
			LCD_botLine("   try again    ");
			__WFI();							// press button to continue
		}

		while(!((GPIO->P[1].DIN) & BUTTON)){}	// wait for button release
		}
}

void GPIO_ODD_IRQHandler()						// even pins interrupt handler
{
	if (GPIO->IF & BUTTON)						// check the interrupt source
	{
		GPIO->IFC = BUTTON;						// clear button IF
	}
}

void RTC_IRQHandler(void)						// RTC ISR
{
	if (RTC->IF & RTC_IF_COMP0) 				// check RTC cnannel 0 event flag
	{
		RTC->IFC = RTC_IFC_COMP0;				// clear RTC flag
		(GPIO->P[1]).DOUTSET = LED;				// turn LED on
	}
	if (RTC->IF & RTC_IF_COMP1)					// check RTC cnannel 1 event flag
	{
		RTC->IFC = RTC_IFC_COMP1;				// clear RTC flag
		RTC->CTRL = 0;							// stop RTC
		(GPIO->P[1]).DOUTCLR = LED;				// turn LED off
	}
}
